/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA6 SCI
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra6_sci.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA6_SCI_H_
#define _DV_RA6_SCI_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#if defined(_DV_RA6M5_H_)
#define SCI0_BASE       0x40118000      /* SCI 0 Base address */
#define SCI1_BASE       0x40118100      /* SCI 1 Base address */
#define SCI2_BASE       0x40118200      /* SCI 2 Base address */
#define SCI3_BASE       0x40118300      /* SCI 3 Base address */
#define SCI4_BASE       0x40118400      /* SCI 4 Base address */
#define SCI5_BASE       0x40118500      /* SCI 5 Base address */
#define SCI6_BASE       0x40118600      /* SCI 6 Base address */
#define SCI7_BASE       0x40118700      /* SCI 7 Base address */
#define SCI8_BASE       0x40118800      /* SCI 8 Base address */
#define SCI9_BASE       0x40118900      /* SCI 9 Base address */

#elif defined(_DV_RA6M3_H_)
#define SCI0_BASE       0x40070000      /* SCI 0 Base address */
#define SCI1_BASE       0x40070020      /* SCI 1 Base address */
#define SCI2_BASE       0x40070040      /* SCI 2 Base address */
#define SCI3_BASE       0x40070060      /* SCI 3 Base address */
#define SCI4_BASE       0x40070080      /* SCI 4 Base address */
#define SCI5_BASE       0x400700A0      /* SCI 5 Base address */
#define SCI6_BASE       0x400700C0      /* SCI 6 Base address */
#define SCI7_BASE       0x400700E0      /* SCI 7 Base address */
#define SCI8_BASE       0x40070100      /* SCI 8 Base address */
#define SCI9_BASE       0x40070100      /* SCI 9 Base address */

#elif defined(_DV_RA4M3_H_)
#define SCI0_BASE       0x40118000      /* SCI 0 Base address */
#define SCI1_BASE       0x40118100      /* SCI 1 Base address */
#define SCI2_BASE       0x40118200      /* SCI 2 Base address */
#define SCI3_BASE       0x40118300      /* SCI 3 Base address */
#define SCI4_BASE       0x40118400      /* SCI 4 Base address */
#define SCI9_BASE       0x40118900      /* SCI 9 Base address */

#else
#error "I/O definition file is undefined."
#endif


/* Event number */

#if defined(_DV_RA6M5_H_)
#define EVENT_SCI0_BASE         0x180
#define EVENT_SCI1_BASE         0x186
#define EVENT_SCI2_BASE         0x18C
#define EVENT_SCI3_BASE         0x192
#define EVENT_SCI4_BASE         0x198
#define EVENT_SCI5_BASE         0x19E
#define EVENT_SCI6_BASE         0x1A4
#define EVENT_SCI7_BASE         0x1AA
#define EVENT_SCI8_BASE         0x1B0
#define EVENT_SCI9_BASE         0x1B6

#elif defined(_DV_RA6M3_H_)
#define EVENT_SCI0_BASE         0x174
#define EVENT_SCI1_BASE         0x17A
#define EVENT_SCI2_BASE         0x180
#define EVENT_SCI3_BASE         0x186
#define EVENT_SCI4_BASE         0x18C
#define EVENT_SCI5_BASE         0x192
#define EVENT_SCI6_BASE         0x198
#define EVENT_SCI7_BASE         0x19E
#define EVENT_SCI8_BASE         0x1A4
#define EVENT_SCI9_BASE         0x1AA

#elif defined(_DV_RA4M3_H_)
#define EVENT_SCI0_BASE         0x180
#define EVENT_SCI1_BASE         0x186
#define EVENT_SCI2_BASE         0x18C
#define EVENT_SCI3_BASE         0x192
#define EVENT_SCI4_BASE         0x198
#define EVENT_SCI9_BASE         0x1B6

#endif

#define EVENT_RXI_OFFSET        0
#define EVENT_TXI_OFFSET        1
#define EVENT_TEI_OFFSET        2
#define EVENT_ERI_OFFSET        3
#define EVENT_AM_OFFSET         4


/* Registers */

typedef struct {
    UB  SMR;            /* Serial Mode Register                                */
    UB  BRR;            /* Bit Rate Register                                   */
    UB  SCR;            /* Serial Control Register                             */
    UB  TDR;            /* Transmit Data Register                              */
    UB  SSR;            /* Serial Status Register                              */
    UB  RDR;            /* Receive Data Register                               */
    UB  SCMR;           /* Smart Card Mode Register                            */
    UB  SEMR;           /* Serial Extended Mode Register                       */
    UB  SNFR;           /* Noise Filter Setting Register                       */
    UB  SIMR1;          /* I2C Mode Register 1                                 */
    UB  SIMR2;          /* I2C Mode Register 2                                 */
    UB  SIMR3;          /* I2C Mode Register 3                                 */
    UB  SISR;           /* I2C Status Register                                 */
    UB  SPMR;           /* SPI Mode Register                                   */
    UH  TDRHL;          /* Transmit 9-bit Data Register                        */
    UH  RDRHL;          /* Receive 9-bit Data Register                         */
    UB  MDDR;           /* Modulation Duty Register                            */
    UB  DCCR;           /* Data Compare Match Control Register                 */
    UH  FCR;            /* FIFO Control Register                               */
    UH  FDR;            /* FIFO Data Count Register                            */
    UH  LSR;            /* Line Status Register                                */
    UH  CDR;            /* Compare Match Data Register                         */
    UB  SPTR;           /* Serial Port Register                                */
    UB  ACTR;           /* Adjustment Communication Timing Register            */
    UH  reserved;
    UB  ESMER;          /* Extended Serial Module Enable Register              */
    UB  CR0;            /* Control Register 0                                  */
    UB  CR1;            /* Control Register 1                                  */
    UB  CR2;            /* Control Register 2                                  */
    UB  CR3;            /* Control Register 3                                  */
    UB  PCR;            /* Port Control Register                               */
    UB  ICR;            /* Interrupt Control Register                          */
    UB  STR;            /* Status Register                                     */
    UB  STCR;           /* Status Clear Register                               */
    UB  CF0DR;          /* Control Field 0 Data Register                       */
    UB  CF0CR;          /* Control Field 0 Compare Enable Register             */
    UB  CF0RR;          /* Control Field 0 Receive Data Register               */
    UB  PCF1DR;         /* Primary Control Field 1 Data Register               */
    UB  SCF1DR;         /* Secondary Control Field 1 Data Register             */
    UB  CF1CR;          /* Control Field 1 Compare Enable Register             */
    UB  CF1RR;          /* Control Field 1 Receive Data Register               */
    UB  TCR;            /* Timer Control Register                              */
    UB  TMR;            /* Timer Mode Register                                 */
    UB  TPRE;           /* Timer Prescaler Register                            */
    UB  TCNT;           /* Timer Count Register                                */
} T_RA6_SCI; 


/* Port access structure */

#define REG_SCI0    (*(volatile T_RA6_SCI *)SCI0_BASE)
#define REG_SCI1    (*(volatile T_RA6_SCI *)SCI1_BASE)
#define REG_SCI2    (*(volatile T_RA6_SCI *)SCI2_BASE)
#define REG_SCI3    (*(volatile T_RA6_SCI *)SCI3_BASE)
#define REG_SCI4    (*(volatile T_RA6_SCI *)SCI4_BASE)
#if defined(_DV_RA6M5_H_) || defined(_DV_RA6M3_H_)
#define REG_SCI5    (*(volatile T_RA6_SCI *)SCI5_BASE)
#define REG_SCI6    (*(volatile T_RA6_SCI *)SCI6_BASE)
#define REG_SCI7    (*(volatile T_RA6_SCI *)SCI7_BASE)
#define REG_SCI8    (*(volatile T_RA6_SCI *)SCI8_BASE)
#endif
#define REG_SCI9    (*(volatile T_RA6_SCI *)SCI9_BASE)


/* Bit definition of register */

                            /* FIFO Receive Data Register */
#define SCI_FRDRHL_RDF      0x4000u
#define SCI_FRDRHL_ORER     0x2000u
#define SCI_FRDRHL_FER      0x1000u
#define SCI_FRDRHL_PER      0x0800u
#define SCI_FRDRHL_DR       0x0400u
#define SCI_FRDRHL_MPB      0x0200u

                            /* Transmit Data Register */
#define SCI_FTDRHL_MPBT     0x0200u

                            /* Serial Mode Register */
#define SCI_SMR_CM          0x80u
#define SCI_SMR_CHR         0x40u
#define SCI_SMR_PE          0x20u
#define SCI_SMR_PM          0x10u
#define SCI_SMR_STOP        0x08u
#define SCI_SMR_MP          0x04u
#define SCI_SMR_CKS         0x03u
#define SCI_SMR_CKS_0       0x00u
#define SCI_SMR_CKS_4       0x01u
#define SCI_SMR_CKS_16      0x02u
#define SCI_SMR_CKS_64      0x03u

                            /* Serial Control Register */
#define SCI_SCR_TIE         0x80u
#define SCI_SCR_RIE         0x40u
#define SCI_SCR_TE          0x20u
#define SCI_SCR_RE          0x10u
#define SCI_SCR_MPIE        0x08u
#define SCI_SCR_TEIE        0x04u
#define SCI_SCR_CKE         0x03u

                            /* Serial Status Register */
#define SCI_SSR_TDRE        0x80u
#define SCI_SSR_TDFE        0x80u
#define SCI_SSR_RDRF        0x40u
#define SCI_SSR_RDF         0x40u
#define SCI_SSR_ORER        0x20u
#define SCI_SSR_FER         0x10u
#define SCI_SSR_PER         0x08u
#define SCI_SSR_TEND        0x04u
#define SCI_SSR_MPB         0x02u
#define SCI_SSR_MPBT        0x01u
#define SCI_SSR_DR          0x01u

                            /* SPI Mode Register */
#define SCI_SPMR_CKPH       0x80u
#define SCI_SPMR_CKPOL      0x40u
#define SCI_SPMR_MFF        0x10u
#define SCI_SPMR_CTSPEN     0x08u
#define SCI_SPMR_MSS        0x04u
#define SCI_SPMR_CTSE       0x02u
#define SCI_SPMR_SSE        0x01u

                            /* FIFO Control Register */
#define SCI_FCR_RSTRG       0xF000u
#define SCI_FCR_RTRG        0x0F00u
#define SCI_FCR_TTRG        0x00F0u
#define SCI_FCR_DRES        0x0008u
#define SCI_FCR_TFRST       0x0004u
#define SCI_FCR_RFRST       0x0002u
#define SCI_FCR_FM          0x0001u
#define SCI_FCR_RSTRG_SH    12u
#define SCI_FCR_RTRG_SH     8u
#define SCI_FCR_TTRG_SH     4u

                            /* FIFO Data Count Register */
#define SCI_FDR_T           0x1F00u
#define SCI_FDR_R           0x001Fu
#define SCI_FDR_T_SH        8u
#define SCI_FDR_R_SH        0u

                            /* Linr Status Register */
#define SCI_LSR_PNUM        0x1F00u
#define SCI_LSR_FNUM        0x007Cu
#define SCI_LSR_ORER        0x0001u
#define SCI_LSR_PNUM_SH     8u
#define SCI_LSR_FNUM_SH     2u

                            /* Serial Port Register */
#define SCI_SPTR_ATEN       0x80u
#define SCI_SPTR_ASEN       0x40u
#define SCI_SPTR_TINV       0x20u
#define SCI_SPTR_RINV       0x10u
#define SCI_SPTR_SPB2IO     0x04u
#define SCI_SPTR_SPB2DT     0x02u
#define SCI_SPTR_RXDMON     0x01u

                            /* Adjustment Communication Timing Register */
#define SCI_ACTR_AET        0x80u
#define SCI_ACTR_ATT        0x70u
#define SCI_ACTR_AJD        0x08u
#define SCI_ACTR_AST        0x07u
#define SCI_ACTR_ATT_SH     4u
#define SCI_ACTR_AST_SH     0u

                            /* Control Register 0 */
#define SCI_CR0_BRME        0x08u
#define SCI_CR0_RXDSF       0x04u
#define SCI_CR0_SFSF        0x02u

                            /* Control Register 1 */
#define SCI_CR1_PIBS        0xE0u
#define SCI_CR1_PIBE        0x10u
#define SCI_CR1_CF1DS       0x0Cu
#define SCI_CR1_CF0RE       0x02u
#define SCI_CR1_BFE         0x01u
#define SCI_CR1_PIBS_SH     5u
#define SCI_CR1_CF1DS_SH    2u

                            /* Control Register 2 */
#define SCI_CR2_RTSR        0xC0u
#define SCI_CR2_BCCS        0x30u
#define SCI_CR2_DFCS        0x07u
#define SCI_CR2_RTSR_SH     6u
#define SCI_CR2_BCCS_SH     4u
#define SCI_CR2_DFCS_SH     0u

                            /* Control Register 3 */
#define SCI_CR3_SDST        0x01u

                            /* Port Control Register */
#define SCI_PCR_SHARPS      0x10u
#define SCI_PCR_RXDXPS      0x02u
#define SCI_PCR_TXDXPS      0x01u

                            /* Interrupt Control Register */
#define SCI_ICR_AEDIE       0x20u
#define SCI_ICR_BCDIE       0x10u
#define SCI_ICR_PIBDIE      0x08u
#define SCI_ICR_CF1MIE      0x04u
#define SCI_ICR_CF0MIE      0x02u
#define SCI_ICR_BFDIE       0x01u

                            /* Status Register */
#define SCI_STR_AEDF        0x20u
#define SCI_STR_BCDF        0x10u
#define SCI_STR_PIBDF       0x08u
#define SCI_STR_CF1MF       0x04u
#define SCI_STR_CF0MF       0x02u
#define SCI_STR_BFDF        0x01u

                            /* Status Clear Register */
#define SCI_STCR_AEDCL      0x20u
#define SCI_STCR_BCDCL      0x10u
#define SCI_STCR_PIBDCL     0x08u
#define SCI_STCR_CF1MCL     0x04u
#define SCI_STCR_CF0MCL     0x02u
#define SCI_STCR_BFDCL      0x01u


#ifdef __cplusplus
}
#endif
#endif /* _DV_RA6_SCI_H_ */
