/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA6 SRAM
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra6_sram.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA6_SRAM_H_
#define _DV_RA6_SRAM_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define SRAM_BASE       0x40002000      /* SRAM Base address */


/* registers */

typedef struct {
    UB  PARIOAD;          /* SRAM Parity Error Operation After Detection Register */
    UB  reserved[3];
    UB  SRAMPRCR;         /* SRAM Protection Register                          */
    UB  reserved1[3];
    UB  SRAMWTSC;         /* RAM Wait State Control Register                   */
    UB  reserved2[3];
    UB  SRAMPRCR2;        /* SRAM Protection Register 2                        */
    UB  reserved3[179];
    UB  ECCMODE;          /* ECC Operating Mode Control Register               */
    UB  ECC2STS;          /* ECC 2-Bit Error Status Register                   */
    UB  ECC1STSEN;        /* ECC 1-Bit Error Information Update Enable Register*/
    UB  ECC1STS;          /* ECC 1-Bit Error Status Register                   */
    UB  ECCPRCR;          /* ECC Protection Register                           */
    UB  reserved4[11];
    UB  ECCPRCR2;         /* ECC Protection Register 2                         */
    UB  reserved5[3];
    UB  ECCETST;          /* ECC Test Control Register                         */
    UB  reserved6[3];
    UB  ECCOAD;           /* SRAM ECC Error Operation After Detection Register */
} T_RA6_SRAM; 


/* Port access structure */

#define REG_SRAM      (*(volatile T_RA6_SRAM *)SRAM_BASE)


#ifdef __cplusplus
}
#endif
#endif /* _DV_RA6_SRAM_H_ */
