/*
 *
 *           UART driver header (ITRON type)
 *                  for Kernel of MULti Core Operating System
 *
 *           Copyright (c) 2021-2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       drv_uart.h
 *
 *  Aug/08/2021     Created.
 *  Sep/01/2022     Added hardflow control.
 *
 */

#ifndef _DRV_UART_H_
#define _DRV_UART_H_

/* Definitions of chatacter length                                      */

#define     BLEN8       0       /* 8-bits length                        */
#define     BLEN7       1       /* 7-bits length                        */
#define     BLEN6       2       /* 6-bits length                        */
#define     BLEN5       3       /* 5-bits length                        */

/* Definitions of parity bit                                            */

#define     PAR_NONE    0       /* None parity                          */
#define     PAR_EVEN    1       /* Even parity                          */
#define     PAR_ODD     2       /* Odd                                  */

/* Definitions of stop bit length                                       */

#define     SBIT1       0       /* 1 stop bit                           */
#define     SBIT15      1       /* 1.5 stop bit                         */
#define     SBIT2       2       /* 2 stop bits                          */
#define     SBIT05      3       /* 0.5 stop bits                        */

/* Definitions of configuration data                                    */

typedef struct st_uart_def {
    UINT baoudrate;             /* Baoudrate                            */
    UINT parity;                /* Parity                               */
    UINT len;                   /* Data length                          */
    UINT sbit;                  /* Stop bit                             */
} T_UART_DEF;

/* Definitions of status data                                           */

typedef struct st_uart_ref {
    UINT status;                /* Status                               */
    UINT rbuffer;               /* Received                             */
    UINT sbuffer;               /* Waiting for transmission             */
} T_UART_REF;


/* Definitions of command code                                          */

#define     RST_TXBUF   0x0001u /* Reset Tx data buffer                 */
#define     RST_RXBUF   0x0002u /* Reset Rx data and status Buffer      */
#define     DIS_TX      0x0004u /* Disable transmitter                  */
#define     DIS_RX      0x0008u /* Disable receiver                     */
#define     ENA_TX      0x0010u /* Enabel transmitter                   */
#define     ENA_RX      0x0020u /* Enable receiver                      */
#define     DIS_CTS     0x0040u /* Disable CTS hardflow control         */
#define     DIS_RTS     0x0080u /* Disable RTS hardflow control         */
#define     ENA_CTS     0x0100u /* Enabel CTS hardflow control          */
#define     ENA_RTS     0x0200u /* Enable RTS hardflow control          */


/* Definitions of status code                                           */

#define     UART_EROR   0x0001u /* Rx Buffer Overflow error             */
#define     UART_ERP    0x0002u /* Parity error                         */
#define     UART_ERF    0x0004u /* Framing error                        */
#define     UART_BRK    0x0008u /* line break status                    */
#define     UART_NOISE  0x0010u /* Noise                                */
#define     UART_ENRX   0x2000u /* 1= Rx Enable, 0= Rx Disable          */
#define     UART_ENTX   0x4000u /* 1= Tx Enable, 0= Tx Disable          */
#define     UART_INIT   0x8000u /* 1= Port Initialized                  */

/* Definitions of Channel integration User's API                        */

ER ini_uart(UINT ch, T_UART_DEF *uart_def);
ER ref_uart(UINT ch, T_UART_REF *uart_ref);
ER ctr_uart(UINT ch, UINT cdata);
ER fls_uart(UINT ch, TMO timeout);
ER putc_uart(UINT ch, VB data, TMO timeout);
ER puts_uart(UINT ch, VB *data, UINT *count, TMO timeout);
ER getc_uart(UINT ch, VB *data, UB *sts, TMO timeout);
ER gets_uart(UINT ch, VB *data, UB *sts, UINT *count, TMO timeout);

/* Definitions of each channel User's API.                              */

ER ini_uart0(T_UART_DEF *uart_def);
ER ref_uart0(T_UART_REF *uart_ref);
ER ctr_uart0(UINT cdata);
ER fls_uart0(TMO timeout);
ER putc_uart0(VB data, TMO timeout);
ER puts_uart0(VB *data, UINT *count, TMO timeout);
ER getc_uart0(VB *data, UB *sts, TMO timeout);
ER gets_uart0(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart1(T_UART_DEF *uart_def);
ER ref_uart1(T_UART_REF *uart_ref);
ER ctr_uart1(UINT cdata);
ER fls_uart1(TMO timeout);
ER putc_uart1(VB data, TMO timeout);
ER puts_uart1(VB *data, UINT *count, TMO timeout);
ER getc_uart1(VB *data, UB *sts, TMO timeout);
ER gets_uart1(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart2(T_UART_DEF *uart_def);
ER ref_uart2(T_UART_REF *uart_ref);
ER ctr_uart2(UINT cdata);
ER fls_uart2(TMO timeout);
ER putc_uart2(VB data, TMO timeout);
ER puts_uart2(VB *data, UINT *count, TMO timeout);
ER getc_uart2(VB *data, UB *sts, TMO timeout);
ER gets_uart2(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart3(T_UART_DEF *uart_def);
ER ref_uart3(T_UART_REF *uart_ref);
ER ctr_uart3(UINT cdata);
ER fls_uart3(TMO timeout);
ER putc_uart3(VB data, TMO timeout);
ER puts_uart3(VB *data, UINT *count, TMO timeout);
ER getc_uart3(VB *data, UB *sts, TMO timeout);
ER gets_uart3(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart4(T_UART_DEF *uart_def);
ER ref_uart4(T_UART_REF *uart_ref);
ER ctr_uart4(UINT cdata);
ER fls_uart4(TMO timeout);
ER putc_uart4(VB data, TMO timeout);
ER puts_uart4(VB *data, UINT *count, TMO timeout);
ER getc_uart4(VB *data, UB *sts, TMO timeout);
ER gets_uart4(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart5(T_UART_DEF *uart_def);
ER ref_uart5(T_UART_REF *uart_ref);
ER ctr_uart5(UINT cdata);
ER fls_uart5(TMO timeout);
ER putc_uart5(VB data, TMO timeout);
ER puts_uart5(VB *data, UINT *count, TMO timeout);
ER getc_uart5(VB *data, UB *sts, TMO timeout);
ER gets_uart5(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart6(T_UART_DEF *uart_def);
ER ref_uart6(T_UART_REF *uart_ref);
ER ctr_uart6(UINT cdata);
ER fls_uart6(TMO timeout);
ER putc_uart6(VB data, TMO timeout);
ER puts_uart6(VB *data, UINT *count, TMO timeout);
ER getc_uart6(VB *data, UB *sts, TMO timeout);
ER gets_uart6(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart7(T_UART_DEF *uart_def);
ER ref_uart7(T_UART_REF *uart_ref);
ER ctr_uart7(UINT cdata);
ER fls_uart7(TMO timeout);
ER putc_uart7(VB data, TMO timeout);
ER puts_uart7(VB *data, UINT *count, TMO timeout);
ER getc_uart7(VB *data, UB *sts, TMO timeout);
ER gets_uart7(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart8(T_UART_DEF *uart_def);
ER ref_uart8(T_UART_REF *uart_ref);
ER ctr_uart8(UINT cdata);
ER fls_uart8(TMO timeout);
ER putc_uart8(VB data, TMO timeout);
ER puts_uart8(VB *data, UINT *count, TMO timeout);
ER getc_uart8(VB *data, UB *sts, TMO timeout);
ER gets_uart8(VB *data, UB *sts, UINT *count, TMO timeout);

ER ini_uart9(T_UART_DEF *uart_def);
ER ref_uart9(T_UART_REF *uart_ref);
ER ctr_uart9(UINT cdata);
ER fls_uart9(TMO timeout);
ER putc_uart9(VB data, TMO timeout);
ER puts_uart9(VB *data, UINT *count, TMO timeout);
ER getc_uart9(VB *data, UB *sts, TMO timeout);
ER gets_uart9(VB *data, UB *sts, UINT *count, TMO timeout);


#endif /* _DRV_UART_H_ */
