/*
 *
 *  MULti Core Operating System - MultiLayer Kernel (MULCOS-M)
 *                   User's API & structure definitions
 *
 *               Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name   mulcosx.h
 *
 *  May/15/2024 Created.
 *
 */

#ifndef MULCOSX_H
#define MULCOSX_H

#include "typedef.h"
#include "mulcos_error.h"
#include "arch_config.h"


/*
 *      Definitions of max specification
 */

#define MULCOSX_MAX_TPRI    128
#define MULCOSX_MAX_MPRI    128
#define MULCOSX_MAX_MSG     32767
#define MULCOSX_MAX_VMPL    32767
#if defined(MULCOSX_MAX_LIMIT)
#define MULCOSX_MAX_TASK    MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_EFLG    MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_SEM     MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_DTQ     MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_MPL     MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_MPF     MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_MBX     MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_MTX     MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_MBF     MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_ISR     MULCOSX_MAX_LIMIT
#define MULCOSX_MAX_CYC     MULCOSX_MAX_LIMIT
#else
#define MULCOSX_MAX_TASK    32767
#define MULCOSX_MAX_EFLG    32767
#define MULCOSX_MAX_SEM     32767
#define MULCOSX_MAX_DTQ     32767
#define MULCOSX_MAX_MPL     32767
#define MULCOSX_MAX_MPF     32767
#define MULCOSX_MAX_MBX     32767
#define MULCOSX_MAX_MTX     32767
#define MULCOSX_MAX_MBF     32767
#define MULCOSX_MAX_ISR     32767
#define MULCOSX_MAX_CYC     32767
#endif
#define MULCOSX_MAX_ACT     255
#define MULCOSX_MAX_WUP     255
#define MULCOSX_MAX_SUS     255
#define MULCOSX_BLK_MIN     8
#define MEMORY_ALIGNEMENT   8
#define MEMORY_MPL_ALIGN    16
#define MEMORY_USR_ALIGN    64
#define NAME_LENGTH         16


/*
 *      Definitions Object attribute
 */

#define BIT_8B_COUNT        8
#define BIT_16B_COUNT       16
#define BIT_32B_COUNT       32
#define BIT_SEM_COUNT       16
#define BIT_DTQ_COUNT       16
#define BIT_MBF_COUNT       16
#define BIT_MPL_COUNT       15
#define BIT_MPF_COUNT       15
#define MAX_SEM_COUNT       ((1<<BIT_SEM_COUNT)-1)
#define MAX_DTQ_COUNT       (1<<BIT_DTQ_COUNT)
#define MAX_MBF_COUNT       (1<<BIT_MBF_COUNT)
#define MAX_MPL_COUNT       (1<<BIT_MPL_COUNT)
#define MAX_MPF_COUNT       (1<<BIT_MPF_COUNT)
#define MULCOSX_MSG_SIZE    (1<<BIT_8B_COUNT)

#define ATR_TFIFO       0x00000000
#define ATR_TPRI        0x00010000
#define ATR_INHERIT     0x00020000
#define ATR_CEILING     0x00030000
#define ATR_OR_WAIT     0x00010000
#define ATR_AND_WAIT    0x00000000
#define ATR_MUL_WAIT    0x00020000
#define ATR_SGL_WAIT    0x00000000
#define ATR_MFIFO       0x00000000
#define ATR_MPRI        0x00020000
#define ATR_ACT         0x00020000
#define ATR_CLR         0x00040000
#define ATR_PLOCK       0x00040000
#define ATR_FPU         0x00100000
#define ATR_PRIV        0x00200000
#define ATR_INTHDR      0x00800000
#define ATR_CREATE      0x01000000
#define ATR_OVLAP       0x02000000
#define ATR_FORCE       0x04000000
#define ATR_WAIT        0x08000000
#define ATR_RING        0x10000000
#define ATR_BIVT        0x20000000
#define ATR_NSEC2       0x40000000
#define ATR_NSECC       0x80000000
#define ATR_NSEC        0xC0000000


#define TRUE            1
#define FALSE           0

#define NONSEC_TASK     0x00008000


/*
 *      Definitions Task & Handler Status
 */

#define MULCOSX_S_RUN       0x0001              // 実行状態
#define MULCOSX_S_RDY       0x0002              // 実行可能状態
#define MULCOSX_S_WAI       0x0004              // 待ち状態
#define MULCOSX_S_SUS       0x0008              // 強制待ち状態
#define MULCOSX_S_WAS       0x000C              // 二重待ち状態
#define MULCOSX_S_DMT       0x0010              // 休止状態
#define MULCOSX_S_STC       0x0100              // STCODE起動要求
#define MULCOSX_S_EXI       0x0200              // EXINFO起動要求
#define MULCOSX_S_TRM       0x0800              // 終了要求
#define MULCOSX_S_TRD       0x0C00              // 終了と削除要求
#define MULCOSX_S_PRI       0x1000              // 優先度順キュー接続
#define MULCOSX_S_FIFO      0x0000              // FIFOキュー接続
#define MULCOSX_S_TMQ       0x8000              // 時間監視

#define MULCOSX_P_TSK       0x0001              // タスクの管理ブロック
#define MULCOSX_P_CYC       0x0002              // 周期ハンドラの管理ブロック

                                                // Only Task Control Block
#define MULCOSX_W_SLP       0x0001              // 起床待ち状態
#define MULCOSX_W_DLY       0x0002              // 時間経過待ち状態
#define MULCOSX_W_SEM       0x0004              // セマフォ資源の獲得待ち状態
#define MULCOSX_W_FLG       0x0008              // イベントフラグの待ち状態
#define MULCOSX_W_SDT       0x0010              // データキューの送信待ち状態
#define MULCOSX_W_RDT       0x0020              // データキューの受信待ち状態
#define MULCOSX_W_MBX       0x0040              // メールの受信待ち状態
#define MULCOSX_W_MTX       0x0080              // ミューテックスのロック待ち状態
#define MULCOSX_W_SMBF      0x0100              // メッセージバッファへの送信待ち状態
#define MULCOSX_W_RMBF      0x0200              // メッセージバッファからの受信待ち状態
#define MULCOSX_W_FMPL      0x2000              // 固定長メモリブロックの獲得待ち状態
#define MULCOSX_W_VMPL      0x4000              // 可変長メモリブロックの獲得待ち状態
#define MULCOSX_W_SLP2      0x8000              // 起床待ち状態2



/*
 *      Definitions Cyclic handler Status
 */

#define MULCOSX_CYC_STP     0x0000              // 周期ハンドラは停止中
#define MULCOSX_CYC_STA     0x0001              // 周期ハンドラは起動中
#define MULCOSX_CYC_RUN     0x0002              // 周期ハンドラは動作中

/*
 *      Definitions structure of User's API
 */

typedef struct mulcosx_msg_t {
    struct mulcosx_msg_t    *msgque;            // メッセージヘッダ
} MULCOSX_MSG;

typedef struct mulcosx_msg_pri_t {
    MULCOSX_MSG             msgque;             // メッセージヘッダ
    uint32_t                msgpri;             // メッセージ優先度
} MULCOSX_MSG_PRI;

typedef struct mulcosx_sys_t {
    uint32_t                free_sysmem;        // システムメモリの空きサイズ
    uint32_t                free_usrmem;        // 動的メモリの空きサイズ
    uint32_t                used_task;          // 生成済みタスクの個数
    uint32_t                used_sem;           // 生成済みセマフォの個数
    uint32_t                used_eflg;          // 生成済みイベントフラグの個数
    uint32_t                used_mbox;          // 生成済みメールボックスの個数
    uint32_t                used_dtq;           // 生成済みデータキューの個数
    uint32_t                used_mtex;          // 生成済みミューテックスの個数
    uint32_t                used_mbf;           // 生成済みメッセージバッファの個数
    uint32_t                used_mpf;           // 生成済み固定長メモリプールの個数
    uint32_t                used_mpl;           // 生成済み可変長メモリプールの個数
    uint32_t                used_isr;           // 生成済みISRの個数
    uint32_t                used_cyc;           // 生成済み周期ハンドラの個数
} MULCOSX_SYS;

typedef struct mulcosx_conf_t {
    uint32_t                sysmem_size;        // システムメモリのサイズ
    uint32_t                usrmem_size;        // 動的メモリのサイズ
    uint32_t                max_task;           // タスクの総数
    uint32_t                max_sem;            // セマフォの総数
    uint32_t                max_eflg;           // イベントフラグの総数
    uint32_t                max_mbox;           // メールボックスの総数
    uint32_t                max_dtq;            // データキューの総数
    uint32_t                max_mtex;           // ミューテックスの総数
    uint32_t                max_mbf;            // メッセージバッファの総数
    uint32_t                max_mpf;            // 固定長メモリプールの総数
    uint32_t                max_mpl;            // 可変長メモリプールの総数
    uint32_t                max_isr;            // ISRの総数
    uint32_t                max_cyc;            // 周期ハンドラの総数
} MULCOSX_CONF;

typedef struct mulcosx_refsem_t {
    uint32_t                wait_tid;           // セマフォ待ちのタスクID（先頭）
    uint32_t                sem_count;          // セマフォのカウンタ
    int8_t                  name[16];           // セマフォの名称
} MULCOSX_REFSEM;

typedef struct mulcosx_refflg_t {
    uint32_t                wait_tid;           // イベントフラグ待ちのタスクID（先頭）
    uint32_t                pattern;            // イベントフラグのビットパターン
    int8_t                  name[16];           // イベントフラグの名称
} MULCOSX_REFFLG;

typedef struct mulcosx_refdtq_t {
    uint32_t                swait_tid;          // データ送信待ちのタスクID（先頭）
    uint32_t                rwait_tid;          // データ受信待ちのタスクID（先頭）
    uint32_t                dtq_count;          // データキューのデータ数
    int8_t                  name[16];           // データキューの名称
} MULCOSX_REFDTQ;

typedef struct mulcosx_refmtex_t {
    uint32_t                lock_tid;           // ロック中のタスクID
    uint32_t                wait_tid;           // ロック待ちのタスクID（先頭）
    int8_t                  name[16];           // ミューテックスの名称
} MULCOSX_REFMTX;

typedef struct mulcosx_refmbf_t {
    uint32_t                swait_tid;          // メッセージバッファの送信待ちのタスクID（先頭）
    uint32_t                rwait_tid;          // メッセージバッファの受信待ちのタスクID（先頭）
    uint32_t                msg_count;          // メッセージバッファ中のメッセージ数
    uint32_t                free_size;          // メッセージバッファの空き領域のサイズ
    int8_t                  name[16];           // メッセージバッファの名称
} MULCOSX_REFMBF;

typedef struct mulcosx_reffmpl_t {
    uint32_t                wait_tid;           // メモリブロック待ちのタスクID（先頭）
    uint32_t                mpl_free_num;       // メモリブロックの空き個数
    int8_t                  name[16];           // メモリプールの名称
} MULCOSX_REFFMPL;

typedef struct mulcosx_refvmpl_t {
    uint32_t                wait_tid;           // メモリブロック待ちのタスクID（先頭）
    uint32_t                mpl_blk_free;       // メモリブロックの空きサイズ
    uint32_t                mpl_blk_max;        // メモリブロックの最大サイズ
    int8_t                  name[16];           // メモリプールの名称
} MULCOSX_REFVMPL;

typedef struct mulcosx_refmbx_t {
    uint32_t                wait_tid;           // メッセージ待ちのタスクID（先頭）
    MULCOSX_MSG             *message;           // 先頭メッセージのアドレス
    uint32_t                message_num;        // メールボックスのメッセージ数
    int8_t                  name[16];           // メールボックスの名称
} MULCOSX_REFMBX;

typedef struct mulcosx_refisr_t {
    uint32_t                intno;              // ISRの割込み番号
    void                  (*isrhdr)(uint32_t);  // ISR起動時のアドレス（ISRのメイン関数）
    int8_t                  name[16];           // ISRの名称
} MULCOSX_REFISR;

typedef struct mulcosx_refcyc_t {
    uint32_t                status;             // 周期ハンドラのステータス
    uint32_t                statim;             // 次に周期ハンドラが起動するまでの時間
    uint32_t                cyctim;             // 周期ハンドラの起動間隔
    uint32_t                exinfo;             // 周期ハンドラ起動時の引数
    int8_t                  name[16];           // 周期ハンドラの名称
} MULCOSX_REFCYC;

typedef struct mulcosx_reftsk_t {
    uint32_t                status;             // タスクのステータス
    uint32_t                priority;           // タスクの現在優先度
    uint32_t                base_pri;           // タスクのベース優先度
    uint32_t                waitobj;            // タスクの待ち要因
    uint32_t                obj_id;             // タスクの待ちオブジェクトのID番号
    uint32_t                wuptime;            // タスクがタイムアウトするまでの時間
    uint32_t                act_count;          // タスクの起動要求カウント数
    uint32_t                sus_count;          // タスクのサスペンドカウント数
    uint32_t                wup_count[2];       // タスクの起床要求カウント数
    int8_t                  name[16];           // 周期ハンドラの名称
} MULCOSX_REFTSK;

typedef struct mulcosx_config_t {
    void                  (*sysdwn)(uint32_t);  // システムダウンハンドラのアドレス
    void                  (*sysidl)(void);      // アイドル関数のアドレス
    uint32_t                usrmem_addr;        // ユーザーメモリ領域のアドレス
    uint32_t                usrmem_size;        // ユーザーメモリ領域のサイズ
    uint32_t                max_task;           // タスクの総数
    uint32_t                rsv_task;           // タスクの予約数
    uint32_t                max_sem;            // セマフォの総数
    uint32_t                rsv_sem;            // セマフォの予約数
    uint32_t                max_eflg;           // イベントフラグの総数
    uint32_t                rsv_eflg;           // イベントフラグの予約数
    uint32_t                max_mbox;           // メールボックスの総数
    uint32_t                rsv_mbox;           // メールボックスの予約数
    uint32_t                max_dtq;            // データキューの総数
    uint32_t                rsv_dtq;            // データキューの予約数
    uint32_t                max_mtex;           // ミューテックスの総数
    uint32_t                rsv_mtex;           // ミューテックスの予約数
    uint32_t                max_mbf;            // メッセージバッファの総数
    uint32_t                rsv_mbf;            // メッセージバッファの予約数
    uint32_t                max_mpf;            // 固定長メモリプールの総数
    uint32_t                rsv_mpf;            // 固定長メモリプールの予約数
    uint32_t                max_mpl;            // 可変長メモリプールの総数
    uint32_t                rsv_mpl;            // 可変長メモリプールの予約数
    uint32_t                max_isr;            // ISRの総数
    uint32_t                rsv_isr;            // ISRの予約数
    uint32_t                max_cyc;            // 周期ハンドラの総数
    uint32_t                rsv_cyc;            // 周期ハンドラの予約数
    uint32_t                max_task_pri;       // 最大タスク優先度数
    uint32_t                clk_msec;           // チック時間
    int32_t                 sgi_int_pri;        // 最低割り込み優先度
    int32_t                 sgi_no;             // ディスパッチ要求の割込み番号
    T_CONFIG_ARCH           arch;               // アーキテクチャ依存部
} MULCOSX_CONFIG;


/*
 *      Definitions User's API of MULCOS-X
 */

        /* Task Management Functions */
RTCODE mulcosx_create_taskid(uint32_t task_id, int8_t *tsk_name, void (*task)(uint32_t), uint32_t pri, uint32_t *stk_base, uint32_t stk_size, uint32_t exinfo, uint32_t flags);
RTCODE mulcosx_delete_task(uint32_t task_id, uint32_t flags);
RTCODE mulcosx_start_task(uint32_t task_id, uint32_t stacode, uint32_t flags);
RTCODE mulcosx_activate_task(uint32_t task_id, uint32_t flags);
RTCODE mulcosx_cancel_activate_task(uint32_t task_id, uint32_t flags);
void   mulcosx_exit_task(uint32_t flags);
void   mulcosx_exit_delete_task(uint32_t flags);
RTCODE mulcosx_terminate_task(uint32_t task_id, uint32_t flags);
RTCODE mulcosx_change_priority(uint32_t task_id,  uint32_t pri, uint32_t flags);
RTCODE mulcosx_reference_priority(uint32_t task_id,  uint32_t *pri, uint32_t flags);
RTCODE mulcosx_reference_task(uint32_t task_id,  MULCOSX_REFTSK *reftsk, uint32_t flags);
RTCODE mulcosx_setup_nosec_stack(uint32_t task_id, uint32_t stk_size);

        /* Task Dependent Synchronization Functions */
RTCODE mulcosx_sleep_task(uint32_t timeout, uint32_t flags);
RTCODE mulcosx_wakeup_task(uint32_t task_id, uint32_t flags);
RTCODE mulcosx_cancel_wakeup(uint32_t task_id, uint32_t flags);
#define mulcosx_sleep_task(p1,p2)      mulcosx_sleep_task_x(0, p1, p2)
#define mulcosx_wakeup_task(p1,p2)     mulcosx_wakeup_task_x(0, p1, p2)
#define mulcosx_cancel_wakeup(p1,p2)   mulcosx_cancel_wakeup_x(0, p1, p2)
RTCODE mulcosx_sleep_task_x(uint32_t slpnum, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_wakeup_task_x(uint32_t slpnum, uint32_t task_id, uint32_t flags);
RTCODE mulcosx_cancel_wakeup_x(uint32_t slpnum, uint32_t task_id, uint32_t flags);
RTCODE mulcosx_release_wait(uint32_t task_id, uint32_t flags);
RTCODE mulcosx_suspend_task(uint32_t task_id, uint32_t flags);
RTCODE mulcosx_resume_task(uint32_t task_id, uint32_t flags);
RTCODE mulcosx_delay_task(uint32_t timeout, uint32_t flags);

        /* Synchronization and Communication (Semaphore) Functions */
RTCODE mulcosx_create_semaphoreid(uint32_t sem_id, int8_t *sem_name, uint32_t init_count, uint32_t max_count, uint32_t flags);
RTCODE mulcosx_delete_semaphore(uint32_t sem_id, uint32_t flags);
RTCODE mulcosx_wait_semaphore(uint32_t sem_id, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_signal_semaphore(uint32_t sem_id, uint32_t flags);
RTCODE mulcosx_reference_semaphore(uint32_t sem_id, MULCOSX_REFSEM *refsem_data, uint32_t flags);

        /* Synchronization and Communication (Eventflags) Functions */
RTCODE mulcosx_create_eflagid(uint32_t efl_id, int8_t *efl_name, uint32_t ini_flagptn, uint32_t flags);
RTCODE mulcosx_delete_eflag(uint32_t eflg_id, uint32_t flags);
RTCODE mulcosx_set_eflag(uint32_t eflg_id, uint32_t pattern, uint32_t flags);
RTCODE mulcosx_wait_eflag(uint32_t eflg_id, uint32_t pattern, uint32_t *get_pattern, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_clear_eflag(uint32_t eflg_id, uint32_t pattern, uint32_t flags);
RTCODE mulcosx_reference_eflag(uint32_t eflg_id, MULCOSX_REFFLG *refflg_data, uint32_t flags);

        /* Synchronization and Communication (Data Queues) Functions */
RTCODE mulcosx_create_dtqid(uint32_t dtq_id, int8_t *dtq_name, void *dtq_adr, uint32_t dtq_count, uint32_t flags);
RTCODE mulcosx_delete_dtq(uint32_t dtq_id, uint32_t flags);
RTCODE mulcosx_send_dtq(uint32_t dtq_id, int32_t data, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_receive_dtq(uint32_t dtq_id, int32_t *data, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_reference_dtq(uint32_t dtq_id, MULCOSX_REFDTQ *refdtq_data, uint32_t flags);

        /* Synchronization and Communication (Mailboxes) Functions */
RTCODE mulcosx_create_mboxid(uint32_t mbx_id, int8_t *mbx_name, uint32_t mbxpri, uint32_t msgmax, uint32_t flags);
RTCODE mulcosx_delete_mbox(uint32_t mbx_id, uint32_t flags);
RTCODE mulcosx_send_mbox(uint32_t mbx_id, MULCOSX_MSG *message, uint32_t flags);
RTCODE mulcosx_receive_mbox(uint32_t mbx_id, MULCOSX_MSG **message, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_reference_mbox(uint32_t mbx_id, MULCOSX_REFMBX *refmbx_data, uint32_t flags);

        /* Extended Synchronization and Communication (MessageBuffer) Functions */
RTCODE mulcosx_create_mbfid(uint32_t mbf_id, int8_t *mbx_name, void *mbf_adr, uint32_t mbf_size, uint32_t max_msg, uint32_t flags);
RTCODE mulcosx_delete_mbf(uint32_t mbf_id, uint32_t flags);
RTCODE mulcosx_receive_mbf(uint32_t mbf_id, uint8_t *msg_buf, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_send_mbf(uint32_t mbf_id, uint8_t *msg_buf, uint32_t msg_size, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_reference_mbf(uint32_t mbf_id, MULCOSX_REFMBF *refmbf_data, uint32_t flags);

        /* Extended Synchronization and Communication (Mutex) Functions */
RTCODE mulcosx_create_mutexid(uint32_t mtx_id, int8_t *mtx_name, uint32_t ceil_pri, uint32_t flags);
RTCODE mulcosx_delete_mutex(uint32_t mtx_id, uint32_t flags);
RTCODE mulcosx_lock_mutex(uint32_t mtx_id, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_unlock_mutex(uint32_t mtx_id, uint32_t flags);
RTCODE mulcosx_reference_mutex(uint32_t mtx_id, MULCOSX_REFMTX *refmtx_data, uint32_t flags);

        /* Memory Pool Management (Fixed-Sized Memory Pools) Functions */
RTCODE mulcosx_create_fmplid(uint32_t fmpl_id, int8_t *fmpl_name, void *pool_adr, uint32_t pool_size, uint32_t blk_size, uint32_t blk_num, uint32_t flags);
RTCODE mulcosx_delete_fmpl(uint32_t mpl_id, uint32_t flags);
RTCODE mulcosx_release_fmpl(uint32_t mpl_id, void *mpl_blk, uint32_t flags);
RTCODE mulcosx_get_fmpl(uint32_t mpl_id, void **mpl_blk, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_reference_fmpl(uint32_t mpl_id, MULCOSX_REFFMPL *reffmpl_data, uint32_t flags);

        /* Memory Pool Management (Variable-Sized Memory Pools) Functions */
RTCODE mulcosx_create_vmplid(uint32_t vmpl_id, int8_t *vmpl_name, void *pool_adr, uint32_t pool_size, uint32_t blk_num, uint32_t flags);
RTCODE mulcosx_delete_vmpl(uint32_t mpl_id, uint32_t flags);
RTCODE mulcosx_release_vmpl(uint32_t mpl_id, void *mpl_blk, uint32_t flags);
RTCODE mulcosx_get_vmpl(uint32_t mpl_id, void **mpl_blk, uint32_t mpl_size, uint32_t flags, uint32_t timeout);
RTCODE mulcosx_reference_vmpl(uint32_t mpl_id, MULCOSX_REFVMPL *refvmpl_data, uint32_t flags);

        /* Time Management (System Time) Functions */
RTCODE mulcosx_supply_tick(void);
RTCODE mulcosx_reference_system_time(uint64_t *systime, uint32_t flags);
RTCODE mulcosx_update_system_time(uint64_t systime, uint32_t flags);

        /* Time Management (Cyclic Handler) Functions */
RTCODE mulcosx_create_cyclicid(uint32_t cyc_id, int8_t *cyc_name, void (*cychdr)(uint32_t), uint32_t cyctim, uint32_t cycphs, uint32_t exinfo, uint32_t flags);
RTCODE mulcosx_delete_cyclic(uint32_t cyc_id, uint32_t flags);
RTCODE mulcosx_start_cyclic(uint32_t cyc_id, uint32_t flags);
RTCODE mulcosx_stop_cyclic(uint32_t cyc_id, uint32_t flags);
RTCODE mulcosx_reference_cyclic(uint32_t cyc_id, MULCOSX_REFCYC *refcyc_data, uint32_t flags);

        /* System State Management Functions */
RTCODE mulcosx_rotate_ready_queue(uint32_t pri, uint32_t flags);
RTCODE mulcosx_reference_cur_task(uint32_t flags);
RTCODE mulcosx_reference_ctex_sts(uint32_t flags);
RTCODE mulcosx_enable_dispatch(uint32_t flags);
RTCODE mulcosx_disable_dispatch(uint32_t flags);
RTCODE mulcosx_reference_dispatch(uint32_t flags);
RTCODE mulcosx_change_IPR(uint32_t intpri, uint32_t flags);
RTCODE mulcosx_reference_IPR(uint32_t flags);
RTCODE mulcosx_disable_IRQ(uint32_t flags);
RTCODE mulcosx_enable_IRQ(uint32_t flags);
RTCODE mulcosx_reference_IRQ(uint32_t flags);
RTCODE mulcosx_reference_delayed_dispatch(uint32_t flags);
RTCODE mulcosx_reference_system_status(MULCOSX_SYS *refsys_data, uint32_t flags);
RTCODE mulcosx_reference_system_config(MULCOSX_CONF *refcfg_data, uint32_t flags);
void mulcosx_enable_vfp(void);
void mulcosx_disable_vfp(void);

        /* Interrupt Management Functions */
RTCODE mulcosx_define_interrupt(uint32_t intno, void (*inthdr)(void), uint32_t intpri, uint32_t flags);
RTCODE mulcosx_enable_interrupt(uint32_t intno, uint32_t flags);
RTCODE mulcosx_disable_interrupt(uint32_t intno, uint32_t flags);
RTCODE mulcosx_create_irq_routineid(uint32_t isr_id, int8_t *isr_name, void (*isrhdr)(uint32_t), uint32_t intno, uint32_t intpri, uint32_t exinfo, uint32_t flags);
RTCODE mulcosx_delete_irq_routine(uint32_t isr_id, uint32_t flags);
RTCODE mulcosx_reference_irq_routine(uint32_t isr_id, MULCOSX_REFISR *refisr_data, uint32_t flags);

        /* MULCOS-T dependent Functions */
uint32_t _mulcosx_get_spi_num(void);

RTCODE mulcosx_init(MULCOSX_CONFIG *mulcosx_cfg);
RTCODE mulcosx_init_sub(uint32_t layer_no, MULCOSX_CONFIG *mulcosx_cfg);
void mulcosx_start(void);
void mulcosx_start_sub(void);


/*
 *      Definitions User's API of virtual memory managemnet
 */

RTCODE mulcosx_get_mem(uint32_t mem_size, void **vir_addr, uint32_t mem_flag);
RTCODE mulcosx_rel_mem(void *vir_addr, uint32_t mem_flag);

extern MULCOSX_CONFIG mulcosx_config;

#endif
