/***********************************************************************
    MULti Core Operating System - MultiLayer kenel (MULCOS-MK)

          Option setting memory for EK-RA8M1 (Setting via debugger)

    Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.

    File name   option_setting.c

    Sep/05/2024 Created.

 ***********************************************************************/


#include "kernel.h"


/***********************************************************************
    Definitions of macro parameter
 ***********************************************************************/

/* No Operation */
#define NO_OPRATION           0xFFFFFFFFul

/* OFS0 : Option Function Select Register 0 */
#define OFS0_INVBIT           0xA001A001ul
#define OFS0_WDT0STPCTL_SH    30
#define OFS0_WDT0RSTIRQS_SH   28
#define OFS0_WDT0PRSS_SH      26
#define OFS0_WDT0PRES_SH      24
#define OFS0_WDT0CKS_SH       20
#define OFS0_WDT0TOPS_SH      18
#define OFS0_WDT0STRT_SH      17
#define OFS0_IWDTSTPCTL_SH    14
#define OFS0_IWDTRSTIRQS_SH   12
#define OFS0_IWDTPRSS_SH      10
#define OFS0_IWDTPRES_SH      8
#define OFS0_IWDTCKS_SH       4
#define OFS0_IWDTTOPS_SH      2
#define OFS0_IWDTSTRT_SH      1

/* DUALSEL : Dual Mode Select Register */
#define DUALSEL_INVBIT        0xFFFFFFF8ul
#define DUALSEL_BANKMD_SH     0

/* SAS : Startup Area Setting Register */
#define SAS_INVBIT            0x7FFF7FFFul
#define SAS_BTFLG_SH          31
#define SAS_FSPR_SH           15

/* OFS1, OFS1_SEC, OFS1_SEL : Option Function Select Register 1 */
#define OFS1_INVBIT           0xFFFFF8F8ul
#define OFS1_HOCOFRQ0_SH      9
#define OFS1_HOCOEN_SH        8
#define OFS1_LVDAS_SH         2
#define OFS1_VDSEL_SH         0

/* BANKSEL, BANKSEL_SEC : Bank Select Register */
#define BANKSEL_INVBIT        0xFF80FFF8ul
#define BANKSEL_BLCKSWP_SH    16
#define BANKSEL_BANKSWP_SH    0

/* Bank mode */
#define BANKMD_DUAL           0
#define BANKMD_LINEAR         7

/* Startup Setting */
#define BTFG_EXCHGL           0u
#define BTFG_NORMAL           1u
#define FSPR_INVALID          0u
#define FSPR_VALID            1u

/* Watchdog Timer Setting */
#define IWDTSTPCTL_CONTI      0u
#define IWDTSTPCTL_STOP       1u
#define IWDTRSTIRQS_INTR      0u
#define IWDTRSTIRQS_RESET     1u
#define IWDTRPSS_25           0u
#define IWDTRPSS_50           1u
#define IWDTRPSS_75           2u
#define IWDTRPSS_100          3u
#define IWDTRPES_75           0u
#define IWDTRPES_50           1u
#define IWDTRPES_25           2u
#define IWDTRPES_0            3u
#define IWDTCKS_DIV1          0u
#define IWDTCKS_DIV16         2u
#define IWDTCKS_DIV32         3u
#define IWDTCKS_DIV64         4u
#define IWDTCKS_DIV128        15u
#define IWDTCKS_DIV256        5u
#define IWDTTOPS_128          0u
#define IWDTTOPS_512          1u
#define IWDTTOPS_1024         2u
#define IWDTTOPS_2048         3u
#define IWDTSTRT_AUTO         0u
#define IWDTSTRT_MANU         1u

/* Watchdog Timer Setting */
#define WDT0STPCTL_CONTI      0u
#define WDT0STPCTL_STOP       1u
#define WDT0RSTIRQS_INTR      0u
#define WDT0RSTIRQS_RESET     1u
#define WDT0RPSS_25           0u
#define WDT0RPSS_50           1u
#define WDT0RPSS_75           2u
#define WDT0RPSS_100          3u
#define WDT0RPES_75           0u
#define WDT0RPES_50           1u
#define WDT0RPES_25           2u
#define WDT0RPES_0            3u
#define WDT0CKS_DIV4          1u
#define WDT0CKS_DIV64         4u
#define WDT0CKS_DIV128        15u
#define WDT0CKS_DIV512        6u
#define WDT0CKS_DIV2048       7u
#define WDT0CKS_DIV8192       8u
#define WDT0TOPS_1024         0u
#define WDT0TOPS_4096         1u
#define WDT0TOPS_8192         2u
#define WDT0TOPS_16384        3u
#define WDT0STRT_AUTO         0u
#define WDT0STRT_MANU         1u

/* Bank Swap */
#define BANKSWP_200000        0u
#define BANKSWP_000000        7u
#define BLCKSWP_PATTERN       0x7Fu

/* Startup configuration setting */
#define HOCOFRQ0_16M          0u
#define HOCOFRQ0_18M          1u
#define HOCOFRQ0_20M          2u
#define HOCOEN_ENABLE         0u
#define HOCOEN_DISABLE        1u
#define LVDAS_RST_ENABLE      0u
#define LVDAS_RST_DISABLE     1u
#define VDSEL_2R94            1u
#define VDSEL_2R87            2u
#define VDSEL_2R80            3u


/***********************************************************************
    Definitions of parameter
 ***********************************************************************/

#if defined(__BUILD_SECURE__)
#define OPTION_SETTING_OFS0         ( OFS0_INVBIT                               | \
                                     (WDT0STPCTL_STOP   << OFS0_WDT0STPCTL_SH)  | \
                                     (WDT0RSTIRQS_RESET << OFS0_WDT0RSTIRQS_SH) | \
                                     (WDT0RPSS_100      << OFS0_WDT0PRSS_SH)    | \
                                     (WDT0RPES_0        << OFS0_WDT0PRES_SH)    | \
                                     (WDT0CKS_DIV128    << OFS0_WDT0CKS_SH)     | \
                                     (WDT0TOPS_16384    << OFS0_WDT0TOPS_SH)    | \
                                     (WDT0STRT_MANU     << OFS0_WDT0STRT_SH)    | \
                                     (IWDTSTPCTL_STOP   << OFS0_IWDTSTPCTL_SH)  | \
                                     (IWDTRSTIRQS_RESET << OFS0_IWDTRSTIRQS_SH) | \
                                     (IWDTRPSS_100      << OFS0_IWDTPRSS_SH)    | \
                                     (IWDTRPES_0        << OFS0_IWDTPRES_SH)    | \
                                     (IWDTCKS_DIV128    << OFS0_IWDTCKS_SH)     | \
                                     (IWDTTOPS_2048     << OFS0_IWDTTOPS_SH)    | \
                                     (IWDTSTRT_MANU     << OFS0_IWDTSTRT_SH))
#define OPTION_SETTING_DUALSEL      ( DUALSEL_INVBIT                            | \
                                     (BANKMD_LINEAR     << DUALSEL_BANKMD_SH))
#define OPTION_SETTING_SAS          ( SAS_INVBIT                                | \
                                     (BTFG_NORMAL       << SAS_BTFLG_SH)        | \
                                     (FSPR_VALID        << SAS_FSPR_SH))

#define OPTION_SETTING_OFS1_SEC     ( OFS1_INVBIT                               | \
                                     (VDSEL_2R80        << OFS1_VDSEL_SH)       | \
                                     (LVDAS_RST_DISABLE << OFS1_LVDAS_SH)       | \
                                     (HOCOEN_DISABLE    << OFS1_HOCOEN_SH)      | \
                                     (HOCOFRQ0_20M      << OFS1_HOCOFRQ0_SH))
#define OPTION_SETTING_BANKSEL_SEC  ( BANKSEL_INVBIT                            | \
                                     (BLCKSWP_PATTERN   << BANKSEL_BLCKSWP_SH)  | \
                                     (BANKSWP_000000    << BANKSEL_BANKSWP_SH))
#define OPTION_SETTING_BPS_SEC0     (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC1     (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC2     (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC3     (NO_OPRATION)
#define OPTION_SETTING_PBPS_SEC0    (NO_OPRATION)
#define OPTION_SETTING_PBPS_SEC1    (NO_OPRATION)
#define OPTION_SETTING_PBPS_SEC2    (NO_OPRATION)

#define OPTION_SETTING_OFS1_SEL     ( OFS1_INVBIT                               | \
                                     (HOCOFRQ0_16M       << OFS1_HOCOFRQ0_SH)   | \
                                     (HOCOFRQ0_18M       << OFS1_HOCOFRQ0_SH)   | \
                                     (HOCOFRQ0_20M       << OFS1_HOCOFRQ0_SH)   | \
                                     (HOCOEN_DISABLE     << OFS1_HOCOEN_SH)     | \
                                     (HOCOEN_ENABLE      << OFS1_HOCOEN_SH))

#define OPTION_SETTING_BANKSEL_SEL  (NO_OPRATION)
#if 1
#define OPTION_SETTING_BPS_SEC0_SEL (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC1_SEL (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC2_SEL (NO_OPRATION)
#define OPTION_SETTING_BPS_SEC3_SEL (NO_OPRATION)
#else
#define OPTION_SETTING_BPS_SEC0_SEL (0x00000000u)
#define OPTION_SETTING_BPS_SEC1_SEL (0xFFFFFFC0u)
#define OPTION_SETTING_BPS_SEC2_SEL (0x0000001Fu)
#define OPTION_SETTING_BPS_SEC3_SEL (0x00000000u)
#endif


#else
#define OPTION_SETTING_OFS1         ( OFS1_INVBIT                               | \
                                     (VDSEL_2R80        << OFS1_VDSEL_SH)       | \
                                     (LVDAS_RST_DISABLE << OFS1_LVDAS_SH)       | \
                                     (HOCOEN_DISABLE    << OFS1_HOCOEN_SH)      | \
                                     (HOCOFRQ0_20M      << OFS1_HOCOFRQ0_SH))
#define OPTION_SETTING_BANKSEL      ( BANKSEL_INVBIT                            | \
                                     (BLCKSWP_PATTERN   << BANKSEL_BLCKSWP_SH)  | \
                                     (BANKSWP_000000    << BANKSEL_BANKSWP_SH))
#define OPTION_SETTING_BPS0         (NO_OPRATION)
#define OPTION_SETTING_BPS1         (NO_OPRATION)
#define OPTION_SETTING_BPS2         (NO_OPRATION)
#define OPTION_SETTING_BPS3         (NO_OPRATION)
#define OPTION_SETTING_PBPS0        (NO_OPRATION)
#define OPTION_SETTING_PBPS1        (NO_OPRATION)
#define OPTION_SETTING_PBPS2        (NO_OPRATION)
#endif


/***********************************************************************
    Definitions of configuration data
 ***********************************************************************/

#ifdef __GNUC__

extern const UW __tz_FLASH_N;
extern const UW __FLASH_NSC_START;
extern const UW __tz_DATA_FLASH_N;
extern const UW __tz_RAM_N;
extern const UW __RAM_NSC_START;

#if defined(__BUILD_SECURE__)
const UW __attribute__((section(".option_setting_ofs0")))         g_ofs0         = OPTION_SETTING_OFS0;
const UW __attribute__((section(".option_setting_dualsel")))      g_dualsel      = OPTION_SETTING_DUALSEL;
const UW __attribute__((section(".option_setting_sas")))          g_sas          = OPTION_SETTING_SAS;

const UW __attribute__((section(".option_setting_ofs1_sec")))     g_ofs1_sec     = OPTION_SETTING_OFS1_SEC;
const UW __attribute__((section(".option_setting_banksel_sec")))  g_banksel      = OPTION_SETTING_BANKSEL_SEC;
const UW __attribute__((section(".option_setting_bps_sec0")))     g_bps_sec0     = OPTION_SETTING_BPS_SEC0;
const UW __attribute__((section(".option_setting_bps_sec1")))     g_bps_sec1     = OPTION_SETTING_BPS_SEC1;
const UW __attribute__((section(".option_setting_bps_sec2")))     g_bps_sec2     = OPTION_SETTING_BPS_SEC2;
const UW __attribute__((section(".option_setting_bps_sec3")))     g_bps_sec3     = OPTION_SETTING_BPS_SEC3;
const UW __attribute__((section(".option_setting_pbps_sec0")))    g_pbps_sec0    = OPTION_SETTING_PBPS_SEC0;
const UW __attribute__((section(".option_setting_pbps_sec1")))    g_pbps_sec1    = OPTION_SETTING_PBPS_SEC1;
const UW __attribute__((section(".option_setting_pbps_sec2")))    g_pbps_sec2    = OPTION_SETTING_PBPS_SEC2;

const UW __attribute__((section(".option_setting_ofs1_sel")))     g_ofs1_sel     = OPTION_SETTING_OFS1_SEL;
const UW __attribute__((section(".option_setting_banksel_sel")))  g_banksel_sel  = OPTION_SETTING_BANKSEL_SEL;
const UW __attribute__((section(".option_setting_bps_sec0_sel"))) g_bps_sec0_sel = OPTION_SETTING_BPS_SEC0_SEL;
const UW __attribute__((section(".option_setting_bps_sec1_sel"))) g_bps_sec1_sel = OPTION_SETTING_BPS_SEC1_SEL;
const UW __attribute__((section(".option_setting_bps_sec2_sel"))) g_bps_sec2_sel = OPTION_SETTING_BPS_SEC2_SEL;
const UW __attribute__((section(".option_setting_bps_sec3_sel"))) g_bps_sec3_sel = OPTION_SETTING_BPS_SEC3_SEL;
#else
const UW __attribute__((section(".option_setting_ofs1")))         g_ofs1_sec     = OPTION_SETTING_OFS1;
const UW __attribute__((section(".option_setting_banksel")))      g_banksel      = OPTION_SETTING_BANKSEL;
const UW __attribute__((section(".option_setting_bps0")))         g_bps_sec0     = OPTION_SETTING_BPS0;
const UW __attribute__((section(".option_setting_bps1")))         g_bps_sec1     = OPTION_SETTING_BPS1;
const UW __attribute__((section(".option_setting_bps2")))         g_bps_sec2     = OPTION_SETTING_BPS2;
const UW __attribute__((section(".option_setting_bps3")))         g_bps_sec3     = OPTION_SETTING_BPS3;
const UW __attribute__((section(".option_setting_pbps0")))        g_pbps_sec0    = OPTION_SETTING_PBPS0;
const UW __attribute__((section(".option_setting_pbps1")))        g_pbps_sec1    = OPTION_SETTING_PBPS1;
const UW __attribute__((section(".option_setting_pbps2")))        g_pbps_sec2    = OPTION_SETTING_PBPS2;
#endif

#endif

/* end */
